#! /usr/central/bin/wish -f
###GGRTixInit       
#--------------------------------------------------------------------#
######################################################################
# Creates a toplevel window - .can                                   #
######################################################################
proc Create_Toplevel {} {

# Create a new toplevel window to hold menus, a canvas and a message widget
    toplevel .can
    wm minsize .can 550 400
    wm geometry .can 700x600
    wm title .can "Map_Marker"
    wm withdraw .
}

#--------------------------------------------------------------------#
######################################################################
# Creates the main-widgets in the toplevel.                          #
# 1. Menu-bar   - frame (.can.menu)                                  #
# 2. Canvas     - canvas (.can.c1)                                   #
# 3. Scrollbars - to view canvas in both x and y                     #
#                 (.can.c1.sx & .can.c1.sy)                          #
# 4. Frame      - to display a message in (.can.mess)                #
# 4. Message    - message (.can.mess.m)                              #
######################################################################
proc Create_Main_Widgets {} {

# Create a frame in .can for the menu options
     frame .can.menu -geometry 20x20

# Create a canvas to display the bitmap/photo and any shapes
    canvas .can.c1 -relief sunken -scrollregion {0 0 1000 1000}
# Create the scrollbar widgets to view the canvas in both x and y
    scrollbar .can.c1.sy -command ".can.c1 yview" -relief sunk
    scrollbar .can.c1.sx -orient horiz -command ".can.c1 xview" -relief sunk
# Pack the scrollbars in the relevant positions
    pack append .can.c1 .can.c1.sx {bottom fillx} .can.c1.sy {right filly}
    .can.c1 config -xscroll ".can.c1.sx set"
    .can.c1 config -yscroll ".can.c1.sy set"

# A frame to display a message in
    frame .can.mess -height 50

# The default message to display
    set message_string "\n\n <A default message>"

    message .can.mess.m -text $message_string -aspect 750
# Pack the message in the message-box
    pack append .can.mess .can.mess.m {frame w}

    pack append .can .can.menu {top fillx pady 4} \
	        .can.c1 {top expand fill} \
	        .can.mess {top fillx pady 4}
}

#----------------
######################################################################
# Create a series of menu buttons in the menu-bar.                   #
#                                                                    #
# args :                                                             #
#       1. List containing titles for the menu-buttons.              #
######################################################################
proc Create_Menu_Buttons {menuList} {

    global env BROWSER

# CREATE THE MENU-BUTTONS
# Loop through list and create each menu-button and pack it
    foreach title $menuList {
        set pathName [string tolower $title]
        menubutton .can.menu.$pathName -text $title \
            -menu .can.menu.$pathName.men
# Also create the menu widget
        menu .can.menu.$pathName.men
# If this is one of the help menus, pack it on the right, else on the left
        if {![string match *Help $title]} {
            pack append .can.menu .can.menu.$pathName {left padx 6}
       } else {
           pack append .can.menu .can.menu.$pathName {right padx 6}
       }
    }

#============================================================================#
# The remaining part of this procedure sets variables to hold the relevant   #
# information to be parsed to procedures to create the "leaf-widgets"        #
#============================================================================#
                                                                                
# Set vars for file-menu and call procedure to create commands
    set pathName ".can.menu.file.men"
    set fileMenuNames [list New Load_Image Load_Map Save_Map Save_Map_As sep \
        Default_HTML sep Quit]
    set fileMenuCommmands [list {.can.c1 delete all; set imagefile ""} \
        {Load_image} {Read_mapfile} {Save_Map_To_File $mapfile} \
        {Save_Map_To_File ""} {null} {getDefaultHTML} {null} {exitRoutine}]
    Add_Commands_toMenu $pathName $fileMenuNames $fileMenuCommmands

# Set vars for options-menu and call procedure to create commands,
# followed by procedure to create checkbuttons, then a cascade
    set pathname ".can.menu.options.men"
    set optionCommandNames [list {Annotate Selected} \
        {Select First Unannotated}]
    set optionMenuCommands [list {Annotate_Selected} \
        {
            set i [SelectFirstUnannotated]
            if {$i != -1} {
                Auto_Select item$i
                postMessage "The first unnannotated object has been selected"
                Annotate_Selected
            }
        }]
    Add_Commands_toMenu $pathname $optionCommandNames $optionMenuCommands

    set optionsCheckList [list Force_Annotation sep Auto_Raise Auto__html \
        Auto__Save sep]
    Add_Checkbuttons_toMenu $pathname $optionsCheckList

    set cascadeName "Grid_Over_Picture"
    Add_Cascade_toMenu $pathname $cascadeName
# Create the cascade menu (consists of radiobuttons)
    set newPath "$pathname.[string tolower $cascadeName]"
    set nameList [list "No Grid" "Grid 25" "Grid 50"]
    set varR "_CHEX"
    set chexVals [list 0 1 2]
    set rButtonCommands [list {Do_Grid 0} {Do_Grid 25} {Do_Grid 50}]
    Add_Radiobuttons_toMenu $newPath $nameList $varR $chexVals $rButtonCommands

# Set vars for create-shapes-menu and call procedure to create commands,
# followed by radiobuttons
    set pathname ".can.menu.create.men"
    set commandName "Default sep"
    set command [list {set _CHOICE ""; Control ""} \
        ""]
    Add_Commands_toMenu $pathname $commandName $command

    set nameList [list Circle_rad Circle_bb Rectangle Polygon]
    set varR "_CHOICE"
    set choiceVals [list CIRCLE_RAD CIRCLE_BB RECTANGLE POLYGON]
    set rButtonCommands [list {Control $_CHOICE} {Control $_CHOICE} \
        {Control $_CHOICE} {Control $_CHOICE}]
    Add_Radiobuttons_toMenu $pathname $nameList $varR $choiceVals \
        $rButtonCommands

# Set vars for edit-menu and call procedure to create commands
# followed by radiobuttons
    set pathname ".can.menu.edit.men"
    set commandNames "Delete_Selected"
    set commandList {Delete_Selected_Shape}
    Add_Commands_toMenu $pathname $commandNames $commandList

    set nameList [list Copy_Selected sep Flip_About-x Flip_About-y sep\
        Move_Point Add_Point Delete_Point]
    set varR "_CHOICE"
    set choiceVals [list COPY_SELECTED "" FLIP_ABOUT-X FLIP_ABOUT-Y "" \
        MOVE_POINT ADD_POINT DELETE_POINT]
    set rButtonCommands [list {Control $_CHOICE} "" {Control $_CHOICE} \
        {Control $_CHOICE} "" {Control $_CHOICE} {Control $_CHOICE} \
        {Control $_CHOICE}]
    Add_Radiobuttons_toMenu $pathname $nameList $varR $choiceVals \
        $rButtonCommands
    set commandNames [list sep "Raise Selected" "Lower Selected"]
    set commandList [list "" {.can.c1 raise selected all} \
        {LowerSelected $imagefile}]
    Add_Commands_toMenu $pathname $commandNames $commandList

# Set vars for attributes-menu and call procedure to create cascades
# followed immediately by radiobuttons for each cascade-menu.
    set pathname ".can.menu.attributes.men"
    set cascadeName "Style"
    Add_Cascade_toMenu $pathname $cascadeName

    set newPath "$pathname.[string tolower $cascadeName]"
    set nameList [list "Fill" "Outline"]
    set varR "_STYLE"
    set styleVals [list fill outline]
    set rButtonCommands [list Change_Selected_Appearance \
        Change_Selected_Appearance]
    Add_Radiobuttons_toMenu $newPath $nameList $varR $styleVals \
        $rButtonCommands

    set cascadeName "Colour"
    Add_Cascade_toMenu $pathname $cascadeName

    set newPath "$pathname.[string tolower $cascadeName]"
    set nameList [list Black White Red Orange Yellow Green Blue Pink \
        Violet Purple]
    set varR "_COLOUR"
    set colourVals [list black white red orange yellow green blue pink \
        violet purple]
    set rButtonCommands [list Change_Selected_Appearance \
        Change_Selected_Appearance Change_Selected_Appearance \
        Change_Selected_Appearance Change_Selected_Appearance \
        Change_Selected_Appearance Change_Selected_Appearance \
        Change_Selected_Appearance Change_Selected_Appearance \
        Change_Selected_Appearance ]
    Add_Radiobuttons_toMenu $newPath $nameList $varR $colourVals \
        $rButtonCommands

    set cascadeName "Line_Width"
    Add_Cascade_toMenu $pathname $cascadeName

    set newPath "$pathname.[string tolower $cascadeName]"
    set nameList [list 1 2 3 4 5]
    set varR "_LINE_WIDTH"
    set widthVals [list 1 2 3 4 5]
    set rButtonCommands [list Change_Selected_Appearance \
        Change_Selected_Appearance Change_Selected_Appearance \
        Change_Selected_Appearance Change_Selected_Appearance]
    Add_Radiobuttons_toMenu $newPath $nameList $varR $widthVals \
        $rButtonCommands

# Create a button allowing the url to be tested.  State becomes normal when
# there is a selected object, that has a URL written to it.
    button .can.menu.test -text "Test_URL" -command Test_Selected_url \
        -state disabled
    pack .can.menu.test -side left -padx 6

# Set vars for html-help-menu and call procedure to create commands
    set pathname ".can.menu.html_help.men"
    set commandNames "$env(MM_BROWSER)"
    set commandList [list {
        set msg "

Spawning $env(MM_BROWSER)"
        Set_Message $msg
        exec $env(MM_BROWSER) $env(MAP_MARKER)/doc_html/HOME.html &
        after 2500
        set msg "

<A default message>"
        Set_Message $msg
        }]
    Add_Commands_toMenu $pathname $commandNames $commandList

# Set vars for help-menu and call procedure to create commands
    set pathname ".can.menu.help.men"
    set commandNames [list About Help]
    set commandList [list "" Help_file]
# Call procedure to create the commands
    Add_Commands_toMenu $pathname $commandNames $commandList
}

#--------------------------------------------------------------------#
######################################################################
# Called when adding command-widgets to a menu-widget.               #
#                                                                    #
# args :                                                             #
#       1. Full pathname for the widget to add commands to.          #
#       2. List containing labels for the commands.                  #
#       3. List containing values for the "-command" option.         #
######################################################################

proc Add_Commands_toMenu {pathname commandNames widgetCommands} {

    set counter 0

    foreach command $commandNames {
        if {$command == "sep"} {
            $pathname add separator
            incr counter
        } else {
###            if {[string match KILL* $command]} {
###                $pathname add command -label $command \
###                    -command [lindex $widgetCommands $counter] \
###                    -state disabled
###            } else {
                $pathname add command -label $command \
                    -command [lindex $widgetCommands $counter]
###            }
            incr counter
        }
    }
}

#--------------------------------------------------------------------#
######################################################################
# Called when adding checkbutton-widgets to a menu-widget.           #
#                                                                    #
# args :                                                             #
#       1. Full pathname for the widget to add checkbuttons to.      #
#       2. List containing labels for the checkbuttons.              #
######################################################################
proc Add_Checkbuttons_toMenu {pathname checkbuttonNames} {

    foreach command $checkbuttonNames {
        if {$command == "sep"} {
            $pathname add separator
        } else {
            if {[string match Auto__* $command]} {
                .can.menu.options.men add checkbutton -label $command \
                    -variable _$command -state disabled
            } else {
                $pathname add checkbutton -label $command -variable _$command
            }
        }
    }
}
    

#--------------------------------------------------------------------#
######################################################################
# Called when adding radiobutton-widgets to a menu-widget.           #
#                                                                    #
# args :                                                             #
#       1. Full pathname for the widget to add radiobuttons to.      #
#       2. List containing labels for the radiobuttons.              #
#       3. Variable containing the name of the global variable       #
#          linked to the radiobuttons.                               #
#       4. List containing values for the global variable.           #
#       5. List containing values for the "-command" option.         #
######################################################################
proc Add_Radiobuttons_toMenu {pathname rButtonNames rButtonVar varVals
    rButtonCommands} {

    set counter 0

    foreach name $rButtonNames {
        if {$name == "sep"} {
            $pathname add separator
            incr counter
        } else {
            $pathname add radiobutton -label $name \
                -variable $rButtonVar \
                -value [lindex $varVals $counter] \
                -command [lindex $rButtonCommands $counter]
            incr counter
        }
    }
}
#--------------------------------------------------------------------#
######################################################################
# Called when adding a cascade-menu-widget to a menu-widget.         #
#                                                                    #
# args :                                                             #
#       1. Full pathname for the widget to add casacde to.           #
#       2. Label for the cascade-menu.                               #
######################################################################
proc Add_Cascade_toMenu {pathname cascadeLabel} {

    $pathname add cascade -label $cascadeLabel \
        -menu $pathname.[string tolower $cascadeLabel]

    menu $pathname.[string tolower $cascadeLabel]
}

#--------------------------------------------------------------------#
######################################################################
# Called when the "Quit" button is invoked.                          #
######################################################################
proc exitRoutine {} {

# Calls procedure "yesNo" and waits for return value.
# If the user selects the "Yes" button, then destroy parent window.
    if {[yesNo "REALLY QUIT?"] == "YES"} {
        destroy .
    }
}

#---------------------
######################################################################
# Create a new toplevel, giving user a message giving the option to  #
# say Yes or No to that option.                                      #
#                                                                    #
# args :                                                             #
#       1. Message string to display in message-widget.              #
######################################################################
proc yesNo {msg} {

    global var

# If a window with this pathname already exists, destroy it.  Eliminates
# the chance of errors.
    if {[winfo exists .yesno]} {
        destroy .yesno
    }

    toplevel .yesno
    message .yesno.msg -text $msg -width 400
    pack .yesno.msg
# Create frame for buttons
    frame .yesno.bottom 
    pack .yesno.bottom -expand yes -fill x
# Create Yes and No buttons setting global "var" accordingly
    button .yesno.bottom.yes -text YES -command {set var YES}
    button .yesno.bottom.no -text NO -command {set var NO}
    pack .yesno.bottom.yes -side left
    pack .yesno.bottom.no -side right
# Wait for a button to be pressed, then destroy the new window
    tkwait variable var
    destroy .yesno
# Return the contents of "var" to the calling procedure
    return $var
}

#-----------------------------------------------------------------------------#
# we are no longer in a procedure "THIS IS THE *MAIN*"                        #
#-----------------------------------------------------------------------------#

# The following if statement checks if the user has the capability to run the
# photo widget.  If not a message is shown telling the user this.
if {[info commands photo] == ""} {
    postMessage "YOU CANNOT RUN THIS VERSION WITH RPPM FILES"
}

global env

# Check for the existance of the environment variables from which the 
# auto_pah is set
if {![info exists env(MAP_MARKER)]} {
    postMessage "Please set environment variable for MAP_MARKER"
    exit
}

lappend auto_path $env(MAP_MARKER)/src_tcl

# what browser to use? you can set your own one here
global BROWSER
if {[info exists env(MM_BROWSER)]} {
    set BROWSER $env(MM_BROWSER)
} else {
    set BROWSER mosaic
}

# Set the globals required throughout mapmarker
global _COLOUR _LINE _STYLE _CHEX _NITEM BROWSER
global input_image_file mapfile imagefile defaulthtml
global objectAnnotations objectURLs _Force_Annotation Auto_Raise

set mapfile ""
set imagefile ""
set defaulthtml ""
set objectAnnotations ""
set objectURLs ""
set _Force_Annotation 1
set _NITEM 0
set _Auto_Raise 1

# Global variables controlling the attributes of any items created in the 
# canvas.  The default is set as a white outline with linewidth=4
set _COLOUR blue
set _LINE_WIDTH 4
set _STYLE outline
# Global for the grid over canvas 0= no grid (set by default)
set _CHEX 0

# Call procedure to create the new toplevel
Create_Toplevel

# Call procedure to create the main-widgets
Create_Main_Widgets

# Hold the titles for each of the menu-buttons as elements of a list to be
# parsed to the procedure to create the menu-buttons
set menuList {File Options Create Edit Attributes Help Html_Help}

# Call procedure to create a series of menubuttons offering options
Create_Menu_Buttons $menuList


