#########################################################################
# This procedure is called when the annotate button in the Miscellaneous#
# menu is pressed.  It gets relevant information about the selected     #
# item and displays it in a series of entry boxes.  These can then be   #
# altered if neccessary by the user, with the option of providing a     #
# small annotation and a URL for the item under observation.            #
#########################################################################
proc Annotate_Selected {} {

# The list of tags for an item contains information about the item.
# Get the list of tags, and identify the item-index. 
    set tagList [.can.c1 gettags selected]

    if {$tagList == ""} {
        postMessage "No object was selected"
        return
    }
    set itemIndex [lindex $tagList 0]
    set elements [llength $tagList]
# From the list of tags get all the required information about the item.
    set attributes [lindex $tagList 1]
    set Comments [lindex $attributes 3]
    set URL [lindex $attributes 4]

# Call procedure allowing user to annotate selected object
    getCommentsAndURL Comments URL

# Amend the "attributes" list to contain the returned values from this
    set attributes [lrange $attributes 0 2]
    lappend attributes $Comments
    lappend attributes $URL

# Remove all the tags from selected item except its index
    for {set a [expr ($elements-1)]} {$a>=1} {incr a -1} {
        set Tag_to_delete [lindex $tagList $a]
        .can.c1 dtag $itemIndex $Tag_to_delete
    }
# And reinsert the tags in the correct order
    .can.c1 addtag $attributes withtag $itemIndex
    .can.c1 addtag selected withtag $itemIndex
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
# Get the comments and URL from the user                              #
#---------------------------------------------------------------------#
proc getCommentsAndURL {Comments URL} {

# Upvar the relevant variables to be parsed back to caller
    upvar 1 $Comments com
    upvar 1 $URL url
# Gain access to global variables for entry-widgets and browser-pid,
# so if "TEST" selected, it can be destroyed when no longer required.
    global comvar urlvar browsepid

# If these vars do not exist, set them to hold null values
    if {![info exists com]} {
        set com ""
    }
    if {![info exists url]} {
        set url ""
    }
# And set the textvariables for the relevant entry-widgets to hold the values
    set comvar $com
    set urlvar $url

    set msg "Annotate the selected object with a comment record \
(written to file) and a URL. The URL (which YOU should start with \
http:// or similar) can be tested by pressing the TEST button which \
activates your selected browser.  Then you can kill the test using \
KILL TEST (NOT quitting the browser)"

# If a window exists with the same pathname, destroy it before the new one 
# is created.--> No errors here.
    if {[winfo exists .comurl]} {
        destroy .comurl
    }

# Create the new toplevel, with a message-widget at top
    toplevel .comurl
    message .comurl.msg -text $msg -width 500
    pack .comurl.msg
# Create a frame containing the label and entry widgets for the URL
    frame .comurl.url
    pack .comurl.url -side top -expand yes -fill x
    label .comurl.url.lab -text URL -width 10 -anchor w
    pack .comurl.url.lab -side left 
    entry .comurl.url.ent -relief sunken
    .comurl.url.ent insert 0 $urlvar
    pack .comurl.url.ent -side left -expand yes -fill x
    bind .comurl.url.ent <Return> {}

# Create a frame containing the label and entry widgets for the comment
    frame .comurl.com
    pack .comurl.com -side top -expand yes -fill x
    label .comurl.com.lab -text Comments -width 10 -anchor w
    pack .comurl.com.lab -side left 
    entry .comurl.com.ent -relief sunken
    .comurl.com.ent insert 0 $comvar
    pack .comurl.com.ent -side left -expand yes -fill x 
    bind .comurl.com.ent <Return> {.comurl.bottom.ok invoke}

# Create a frame containing buttons giving the user options
    frame .comurl.bottom
    pack .comurl.bottom -expand yes -fill x
# Button "OK" bound to user-finished with new values
    button .comurl.bottom.ok -text OK \
        -command {
            set comvar [.comurl.com.ent get]
            set urlvar [.comurl.url.ent get]
            set finished 1
        }
# Button "Cancel" bound to user finished ignore any changes
    button .comurl.bottom.cancel -text CANCEL \
        -command {
            set finished 1
        }
# Button "TEST" bound to create the users chosen browser to test the url.
# Deactivates when selected, reactivates when browser killed.
    button .comurl.bottom.test -text TEST \
        -command {
            set url [.comurl.url.ent get]
            set browsepid [exec $BROWSER $url &]
            .comurl.bottom.test configure -state disabled
            .comurl.bottom.kill configure -state active
        }
# Button "KILL TEST" bound to destroy the browser.  Only active if "TEST" 
# is selected.
    button .comurl.bottom.kill -text "KILL TEST" \
        -command {
            if {$browsepid != ""} {
                exec kill -9 $browsepid 
                .comurl.bottom.test configure -state active
                .comurl.bottom.kill configure -state disabled
            }
        }

    .comurl.bottom.kill configure -state disabled
    pack .comurl.bottom.ok -side left -expand yes -fill x
    pack .comurl.bottom.cancel -side left -expand yes -fill x
    pack .comurl.bottom.test -side left -expand yes -fill x
    pack .comurl.bottom.kill -side left -expand yes -fill x
# Wait until the user is finished, destroy the window and set the return 
# values accordingly.
    tkwait variable finished
    destroy .comurl
    set url $urlvar
    set com "$comvar"
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
# return the serial number of the first unannotated object            #
#---------------------------------------------------------------------#
proc SelectFirstUnannotated {} {

    global _NITEM

# Loop through all items
    for {set i 0} {$i < $_NITEM} {incr i} {
# Get a list of the tags for the item
        set tagList [.can.c1 gettags item$i]
# If this list is an empty string then the item will have been destroyed,
# so continue through loop
        if {$tagList == ""} {
            continue
        }
# If not an empty string, identify if the attributes holds a value for the 
# url. If not then exit the loop returning the index of the unannotated object
        set Attribs [lindex $tagList 1]
        set url [lindex $Attribs 4]
        if {$url == ""} {
            return $i
        }
    }
# If all items have been annotated, then return "-1"
    return -1
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

# gets the URL of the selected item.  If there is none selected or the
# selected has no URL a blank string is returned

proc getURLOfSelected {} {
     
    set url ""
    set tagList [.can.c1 gettags selected]
    if {$tagList != ""} {
        set url [lindex [lindex $tagList 1] 4]
    }
    return $url
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

proc Test_Selected_url {} {

    global env BROWSER 

    set tagList [.can.c1 gettags selected]

    if {$tagList == ""} {
        postMessage "There is currently no selected item"
    } else {
        set url [getURLOfSelected]
        if {$url == ""} {
            postMessage "The selected item has not been fully annotated"
        } else {
            set browsepid [exec $BROWSER $url &]
            set allPaths \
                [list file options create edit attributes html_help help]
            foreach path $allPaths {
                .can.menu.$path configure -state disabled
            }

            postMessage "A browser will be displayed shortly.  When you have \
                finished the test, press OK here, rather than quitting the \
                browser"
            exec kill -9 $browsepid 
            foreach path $allPaths {
                .can.menu.$path configure -state normal
            }
        }
    }
}


