######################################################################
# This procedure is called in one of two ways :                      #
# 1. When a radiobutton is selected it is called with a value.       #
#        Depending on this value, various procedures are called to   #
#        bind handlers to the relevant sequence of events.           #
# 2. After a complete cycle in a particular sequence of events.      #
#        In this case the procedure is called with an empty string.  #
#        This calls the relevant procedures to set up the default    #
#        bindings.                                                   #
#                                                                    #
# Arg : "c" - Contains value of radiobutton variable "_CHOICE" if    #
#              procedure invoked from radiobutton.  Or an empty      #
#              string if call to reset default bindings.             #
######################################################################
proc Control {c} {

# Gain access to global "_NPOINTS"
    global _NPOINTS

# If a selected function has just been executed, this procedure is called
# with an empty string.  In doing this, the additional bindings created for
# a particular function can be overwritten with the more general ones.
    if {$c==""} {
        set msg "\n\n <A default message>"
        Set_Message $msg

# Set these <sequence of events> to invoke null bindings prior to
# setting defaults
        .can.c1 bind selpt <ButtonPress-1> {}
        .can.c1 bind copt <ButtonPress-1> {}
        bind .can.c1 <ButtonRelease-1> {}
        bind .can.c1 <ButtonPress-2> {}

# Call to the procedures to set the general bindings for the canvas
        Scale_Item
        Move_Item
        Select_New

    } else {
# If a function is selected through the radio-buttons, then set the binding
# allowing user to select a new shape to null until function is executed.
        .can.c1 bind item <ButtonPress-1> {}

# If the selection is to create a new shape, then deselect the currently
# selected item.
        case $c in {
            {CIRCLE_RAD CIRCLE_BB OVAL RECTANGLE POLYGON SPLINED} {
                eval ".can.c1 delete boundingbox selpt"
                eval ".can.c1 addtag item withtag selected"
                eval ".can.c1 dtag selected"
            }
        }

# Parameter 'c' contains the value of the 'choice' variable from main.
# Its value determines which procedures are called, which define which
# bindings are activated
        case $c in {
            {CIRCLE_RAD CIRCLE_BB RECTANGLE} {
                if {$c=="CIRCLE_RAD"} {
                    set msg " <BP-1> - Identifies centre-point \
                        \n <B1-M> - to outer edge \n <BR-1> - Create the shape"
                } else {
                    set msg " <BP-1> - Identifies top-left/bottom-right corner \
                        \n <B1-M> - to oppposite corner \
                        \n <BR-1> - Create the shape"
                }
                Set_Message $msg
                Create_Shape_BP1
                Create_Shape_B1M
                Create_Shape_BR1
            }
            {POLYGON} {
                set _NPOINTS 0
                set msg " <Button-1> - Identifies each point \
                    \n <Button-2> - closes and draws shape \n"
                Set_Message $msg
                Create_Poly_BP1
                Create_Poly_BP2
            }
            COPY_SELECTED {
                set msg "Press <Button-1> on selected shape, and drag the \
                    outline to a new position.  Releasing <Button-1> creates \
                    the copy. \n"
                Set_Message $msg

                bind .can.c1 <ButtonPress-1> {}
                Copy_Shape_BP1
                Copy_Shape_B1M
                Copy_Shape_BR1
            }
            FLIP_ABOUT-X {
                set msg "Click <Button-1> on any point on the bounding box of \
                    selected shape.  Flip occurs around the chosen point."
                Set_Message $msg
                Flip_Item
            }
            FLIP_ABOUT-Y {
                set msg " Click <Button-1> on any point on the bounding box of \
                    selected shape.  Flip occurs around the chosen point."
                Set_Message $msg
                Flip_Item
            }
            MOVE_POINT {
                set msg "Press <Button-1> on a coordinate point, drag to new \
                    location and release to reposition point. \n"
                Set_Message $msg
                Create_Co_Points
            }
            ADD_POINT {
                set msg {Press <Button-1> in a point.  The new point will be \
                    inserted "after" the selected point.  Drag to required \
                    position and release.}
                Set_Message $msg
                Create_Co_Points
            }
            DELETE_POINT {
                set msg "Press <Button-1> in a point and it will be deleted. \
                    \n\n"
                Set_Message $msg
                Create_Co_Points
            }
        }
    }
}
