################################################################################
# This is the general help procedure.  It creates a new toplevel window        #
# containing a list of all the available options on one side and a message     #
# widget on the other.  The user can select which option help is required for  #
# then pressing the "Select" button changes the displayed message to the       #
# relevant one.                                                                #
################################################################################
proc Help_file {} {

# Create the new toplevel, set its size and title
    toplevel .help
    wm geometry .help 700x400
    wm title .help Help

# Create 2 frames - one for the listbox-widget and one for the message-widget
    frame .help.f1 -height 300 -width 300
    frame .help.f2 -height 300 -width 400
# Pack these next to eachother in the window
    pack append .help .help.f1 {left filly} \
                      .help.f2 {left filly}

# Create a scrollbar so the user can adjust the displayed portion of the listbox
    scrollbar .help.f1.scroll1 -command ".help.f1.l1 yview"

# Create the listbox and make it so only one item can be selected at a time
    listbox .help.f1.l1 -relief sunken -geometry 25x15\
        -yscroll ".help.f1.scroll1 set"
    tk_listboxSingleSelect .help.f1.l1

# Create the button to select an option for help, and set its command to call
# the procedure to change the displayed message.
    button .help.f1.b1 -relief raised -text Select \
        -command {
# Set a variable to hold the index of the selected item in listbox, and call
# procedure to show the relevant messagein the message-widget.
                set helpChoice [.help.f1.l1 curselection]
                help_file_1 $helpChoice
        }
# Create the button used to remove the window from the screen
    button .help.f1.b2 -relief raised -text OK -command "destroy .help"

# Pack the widgets in the frame on the left
    pack append .help.f1 .help.f1.scroll1 {right filly} \
                         .help.f1.l1 {top expand fill} \
                         .help.f1.b1 {left} \
                         .help.f1.b2 {right}


# Set a list to hold all values of strings to be inserted in the listbox
    set listBoxElements [list New Load_Image Load_Map Save_Map Save_Map_As \
        Default_HTML Quit "Annotate Selected" "Select First Unannotated" \
        Force_Annotation Auto_Raise Auto_html Auto_Save Grid_Over_Picture\
        Default "Create Circle (r)" "Create Circle (bb)" "Create Rectangle" \
        "Create Polygon" Delete_Selected Copy_Selected Flip_About-x \
        Flip_About-y Move_Point Add_Point Delete_Point "Raise Selected" \
        "Lower Selected" Style Colour Line_Width Test_URL]

# Loop through list insert each element into the listbox-widget
    foreach element $listBoxElements {
        .help.f1.l1 insert end $element
    }


# Set the 'default help-message'
    set helpmess "By pressing mouse-button-1 on an item and clicking \
    \"Select\" gets relevant help message.  Click OK when finished"

# Create the message and display it in the frame on the right
    message .help.f2.m1 -text $helpmess -width 390
    pack append .help.f2 .help.f2.m1 {frame nw}
}

#-----------------------------------------------------------------------------#
# This file amends the contents of the variable containing the help-message   #
# string and reconfigures the message-widget to display the new message.      #
# Arg : "listboxIndex - contains the list-item-reference selected by user.    #
#               This is used to set the message to the correct string         #
#-----------------------------------------------------------------------------#
proc help_file_1 {listboxIndex} {

    set newHelpMessage "By pressing mouse-button-1 on an item and clicking \
        select gets relevant help message.  Click OK when finished"
    case $listboxIndex in {
        0 {
# Help message for New
            set newHelpMessage "Clears all objects from the canvas."
        }
        1 {
# Help message for Load_Image
            set newHelpMessage "Brings up a new toplevel window containing an \
                entry-box. \n Type in the full pathname of the image-file \
                (currently *.xbm and *.rppm).  If the image file exists, it \
                will now be loading."
        }
        2 {
# Help message for Load_Map
            set newHelpMessage " Brings up a new toplevel window containing an \
                entry-box.  Type in the full pathname of the map-file (NCSA \
                format only).  If the map-file contains a pointer to an \
                image-file the option will be given to load this also."
        }
        3 {
# Help message for Save_Map
            set newHelpMessage " If the current mapfile has been loaded from a \
                file, or saved previously under a specified name, then the \
                current state of the canvas is saved under that name. \n\
                If this button is pressed with no filename to save to, a window \
                will appear asking for a filename for the mapfile to be \
                inserted in an entrybox.\n\
                An additional request is for a default_html to point to.\n\
                Finally, an extra is that if there is currently an image-file \
                specified, the filename will be saved in the mapfile.  Then if \
                the mapfile is reloaded the option will be offered to reload \
                the image-file also."
        }
        4 {
# Help message for Save_Map_As
            set newHelpMessage " Brings up a new window containing an \
                entry-box.  Insert the name of the file (*.map) you wish to \
                save to and press OK. \n\
                If there is currently no default_html file \
                specified, the option of naming one will be offered.\
                This is done in the same way as described above."
        }
        5 {
# Help message for Default_HTML
            set newHelpMessage "Brings up a new window containing an \
                entry-box.  Insert the name of the default_html file you would \
                like the background of the image to point to and press OK."
        }
        6 {
# Help message for Quit
            set newHelpMessage " Does exactly that.\n\
                Just in case this is accidentally invoked, a small window will \
                appear asking if this is what you really do wish to quit. \n\
                \n Just press Yes or No."
        }
        7 {
# Help message for Annotate Selected
            set newHelpMessage " If this button is pressed and there is \
                currently a selected object on the canvas, a window will be \
                created containing two entry-boxes. \n\
                If the object has previously been annotated to any degree, this \
                information will be contained in the relevant entry-box. If \
                not then the entry-boxes will be empty. \n\
                This can now be edited to hold the required information."
        }
        8 {
# Help message for Select First Unannotated
            set newHelpMessage " If this button is pressed, all objects on the \
                canvas will be checked to see if it holds a value for a URL. \n\
                The first item in the canvas' display-list with no URL-value \
                will automatically be selected, and a message-box will come up \
                to confirm this.  Press OK, then the \"Annotation\" window will \
                be created allowing annotation to be carried out."
        }
        9 {
# Help message for Force_Annotation
            set newHelpMessage " If this checkbutton is switched on, each time \
                a new item is created the \"Annotation\" window will be called. \
                This means that no object can escape being annotated. \n\
                It is switched to the \"ON\" state by default."
        }
        10 {
# Help message for Auto_Raise
            set newHelpMessage " If this checkbutton is switched on, each time \
                a new item is selected it will automatically be raised to the \
                top of the canvas' display list.  In the off position, all \
                objects will hold relative positions in the display list. \n\
                It is switched to the \"ON\" state by default."
        }
        11 {
# Help message for Auto_html
            set newHelpMessage "NOT YET IMPLEMENTED"
        }
        12 {
# Help message for Auto_Save
            set newHelpMessage "NOT YET IMPLEMENTED"
        }
        13 {
# Help message for Grid_Over_Picture
            set newHelpMessage " A cascade menu containing a series of \
                radiobuttons.  The \"No_Grid\" button is set as the default. \n\
                This is intended purely as an aid to help the user in creating \
                objects to completely envelop areas."
        }
        14 {
# Help message for Default
            set newHelpMessage " This resets all canvas bindings to the default \
                values - Select-new, Deselect-current, Move-selected and \
                Scale-selected. \n\
                If a radiobutton is selected to carry out a certain function, \
                and is not required with no action being carried out on the \
                canvas, pressing this button cancels the previous selection."
        }
        15 {
# Help message for Create Circle (r)
            set newHelpMessage "Pressing Mouse-button-1 identifies a \
                point for the centre of the circle.  Dragging in the \
                x-direction increases/decreases size.  Release button \
                to draw the item."
        }
        16 {
# Help message for Create Circle (bb)
            set newHelpMessage "Pressing Mouse-button-1 identifies a point for \
                either the top-left or the bottom-right corner of a bounding \
                box in which the circle will be drawn. Dragging in the \
                x-direction increases/decreases size.  Release button to \
                draw the item."
        }
        17 {
# Help message for Create Rectangle
            set newHelpMessage "Pressing Mouse-button-1 identifies a point for \
                either the top-left or the bottom-right corner of a \
                bounding box in which the rectangle will be drawn. Dragging \
                in the x/y-directions increases/decreases size.  Release \
                button to draw the item."
        }
        18 {
# Help message for Create Polygon
            set newHelpMessage "Pressing Mouse-button-1 puts a point at the \
                location of the cursor.  This is done as many times as \
                required, with the last point being drawn with \
                Mouse-button-2.  This closes the polygon and draws it."
        }
        19 {
# Help message for Delete_Selected
            set newHelpMessage "If there is currently a selected object on the \
                canvas and this button is pressed, the object will be deleted."
        }
        20 {
# Help message for Copy_Selected
            set newHelpMessage " Pressing Mouse-button-1 in the selected shape \
                displays an outline of the shape. This also creates a copy of \
                the shape over the original. \n\
                Dragging with Mouse-button-1 still pressed allows the outline \
                to be dragged across the canvas. \n\
                Releasing Mouse-button-1 calculates the change in position for \
                the copy and moves the copy.  This is only executed if the \
                mouse has been dragged across the canvas.  If no motion has \
                occurred then the copy is deleted."
        }
        21 {
# Help message for Flip_About-x
            set newHelpMessage " This function is carried out by the user \
                pressing Mouse-button-1 in one of the points on the \
                bounding-box of the selected object. \n\
                The flip occurs using the chosen point as a centre for the \
                flip.  With this option, there is no change in the coordinate \
                y-values."
        }
        22 {
# Help message for Flip_About-y
            set newHelpMessage " This function is carried out by the user \
                pressing Mouse-button-1 in one of the points on the \
                bounding-box of the selected object. \n\
                The flip occurs using the chosen point as a centre for the \
                flip.  With this option, there is no change in the coordinate \
                x-values."
        }
        23 {
# Help message for Move_Point
            set newHelpMessage " This is only used on polygon type objects. \n\
                If this button is selected, the bounding-box and its points \
                will be deleted and coordinate points for the shape will be \
                shown. \n\
                Any of these points can be picked up by pressing Mouse-button-1 \
                in it.  Keeping Mouse-button-1 pressed and dragging the cursor \
                across the canvas moves the chosen point with it.  Move it to \
                the required location and release the button to put the point \
                down."
        }
        24 {
# Help message for Add_Point
            set newHelpMessage " This is only used on polygon type objects. \n\
                If this button is selected, the bounding-box and its points \
                will be deleted and coordinate points for the shape will be \
                shown. \n\
                Press Mouse-button-1 in a point creates a point to be inserted \
                after the selected point in the coordinate list of the polygon. \
                Dragging with Mouse-button-1 draws a line connecting the point \
                with the one after it, through the current cursor location. \n\
                Releasing the mouse-button creates the additional point at the \
                current cursor location."
        }
        25 {
# Help message for Delete_Point
            set newHelpMessage " This is only used on polygon type objects. \n\
                If this button is selected, the bounding-box and its points \
                will be deleted and coordinate points for the shape will be \
                shown. \n\
                Pressing Mouse-button-1 in any of the coordinate points will \
                delete that point and redraw the amended polygon. \n\
                NB// A polygon cannot have less than three coordinate points."
        }
        26 {
# Help message for Raise Selected
            set newHelpMessage "Raises the currently selected object above all \
                others on the canvas."
        }
        27 {
# Help message for Lower Selected
            set newHelpMessage "Lowers the currently selected object below all \
                others, to a position just above the photo/bitmap."
        }
        28 {
# Help message for Style
            set newHelpMessage " This is one of the attributes of the objects. \
                \ It controls the style in which an object is displayed on the \
                canvas. \n\
                FILL    - draws the object fully coloured in. \n\
                OUTLINE - draws the object as an outline."
        }
        29 {
# Help message for Colour
            set newHelpMessage " This is one of the attributes of the objects. \
                \n\ It controls the colour in which an object is drawn."
        }
        30 {
# Help message for Linewidth
            set newHelpMessage " This is one of the attributes of the objects. \
                \n\ If an object is drawn in \"Outline\" style, this controls \
                the width of the outline."
        }
        31 {
# Help message for Test_URL
            set newHelpMessage " This button only becomes active if ther is \
                currently a selected object on the canvas and that object has \
                been annotated to give a URL. \n\
                Pressing this button spawns your chosen browser with the file \
                that is pointed to."
        }

    }
# Reconfigure the help message-widget
    .help.f2.m1 configure -text $newHelpMessage -width 390
}
